# GBM BOVM VCVaR Functions.R
#
source("ASGBMBINOVM With SRM Functions.R")
#
# Portfolio risk management functions
#
# Select between European-style and American-style
fBINOptionValue = function(B){
  OV <- -99
  if(B$Style == 1)OV <- ESBINOptionValue(B)
  else if(B$Style == 2)OV <- ASBINOptionValue(B)
  return(OV)
}
# Set initial values for 19 positions
fInitialValues = function(BINInputData, InitialValuations){
  tIV <- InitialValuations
# Long Stock
  tIV["LS", "InitialValue"] <- BINInputData$Units * BINInputData$StockPrice
# Long call  
  BINInputData$Type <- 1L # 1 for call, -1 for put
  tX <- BINInputData$StrikePrice
  tIV["LCX", "InitialValue"] <- BINInputData$Units * fBINOptionValue(BINInputData)
  BINInputData$StrikePrice <- tX*(1 - BINInputData$StrikeIncrement/100)
  tIV["LCXL", "InitialValue"] <- BINInputData$Units * fBINOptionValue(BINInputData)
  BINInputData$StrikePrice <- tX*(1 + BINInputData$StrikeIncrement/100)
  tIV["LCXH", "InitialValue"] <- BINInputData$Units * fBINOptionValue(BINInputData)
  BINInputData$StrikePrice <- tX
# Long put
  BINInputData$Type <- -1L # 1 for call, -1 for put
  tX <- BINInputData$StrikePrice
  tIV["LPX", "InitialValue"] <- BINInputData$Units * fBINOptionValue(BINInputData)
  BINInputData$StrikePrice <- tX*(1 - BINInputData$StrikeIncrement/100)
  tIV["LPXL", "InitialValue"] <- BINInputData$Units * fBINOptionValue(BINInputData)
  BINInputData$StrikePrice <- tX*(1 + BINInputData$StrikeIncrement/100)
  tIV["LPXH", "InitialValue"] <- BINInputData$Units * fBINOptionValue(BINInputData)
  BINInputData$StrikePrice <- tX
# Long covered call writing  
  tIV["LCCWX", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LCX", "InitialValue"]
  tIV["LCCWXL", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LCXL", "InitialValue"]
  tIV["LCCWXH", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LCXH", "InitialValue"]
# Long protective put buying
  tIV["LPPBX", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LPX", "InitialValue"]
  tIV["LPPBXL", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LPXL", "InitialValue"]
  tIV["LPPBXH", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LPXH", "InitialValue"]
# Long leveraged call  
  tIV["LLCX", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LCX", "InitialValue"]
  tIV["LLCXL", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LCXL", "InitialValue"]
  tIV["LLCXH", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LCXH", "InitialValue"]
# Long leveraged put
  tIV["LLPX", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LPX", "InitialValue"]
  tIV["LLPXL", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LPXL", "InitialValue"]
  tIV["LLPXH", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LPXH", "InitialValue"]
  tIV <- round(tIV,4)
  return(tIV)
}
