# ABM BOVM MCVaR Core.R
fABMBOVMMCVaRCore = function(BINInputData, InitialValuations, Variable){
  Horizon = SimulationInputData$Horizon
  NumberOfSecurities <- 1 # Analysis of only one stock
  NumberOfRandomVariables <- 3.0 # Stock, Rates, Volatility
  ConfidenceLevel <- SimulationInputData$ConfidenceLevel
  # Extract Name, Units, Price, Mean, Standard Deviation, and Correlation inputs
  Name = SimulationInputData$Name
  Units = inputUnits
  Price = inputStockPrice1 # Just one stock for now
  Mean = as.numeric(c(1:NumberOfRandomVariables))
  Mean[1] <- SimulationInputData$StockMean/100.0
  Mean[2] <- SimulationInputData$InterestRateMean/100.0
  Mean[3] <- SimulationInputData$SDMean/100.0
  StandardDeviation = as.numeric(c(1:NumberOfRandomVariables))
  StandardDeviation[1] <- SimulationInputData$StockSD/100.0
  StandardDeviation[2] <- SimulationInputData$InterestRateSD/100.0
# SD IN $
  StandardDeviation[3] <- SimulationInputData$SDSD
  Correlation = matrix(data = 0, nrow = NumberOfRandomVariables, ncol = NumberOfRandomVariables)
  Correlation[1,1] <- Correlation[2,2] <- Correlation[3,3] <- 1.0
  Correlation[1,2] <- Correlation[2,1] <- SimulationInputData$CorStockRate
  Correlation[1,3] <- Correlation[3,1] <- SimulationInputData$CorStockVol
  Correlation[2,3] <- Correlation[3,2] <- SimulationInputData$CorRateVol
  Covariance = matrix(nrow = NumberOfRandomVariables, ncol = NumberOfRandomVariables)
  Covariance[1,1] <- StandardDeviation[1]^2
  Covariance[2,2] <- StandardDeviation[2]^2 
  Covariance[3,3] <- StandardDeviation[3]^2 
  Covariance[1,2] <- Covariance[2,1] <- StandardDeviation[1] * StandardDeviation[2] * Correlation[1,2]
  Covariance[1,3] <- Covariance[3,1] <- StandardDeviation[1] * StandardDeviation[3] * Correlation[1,3]
  Covariance[2,3] <- Covariance[3,2] <- StandardDeviation[2] * StandardDeviation[3] * Correlation[2,3]
# Data frame holding outputs
  InitialValuations = data.frame(matrix(NA, nrow = NumberOfStrategies, ncol = NumberOfSecurities))
  colnames(InitialValuations)  <- c("InitialValue")
  rownames(InitialValuations)  <- c("LS", # Long stock
    "LCXL", "LCX", "LCXH", # Long call
    "LPXL", "LPX", "LPXH", # Long put
    "LCCWXL", "LCCWX", "LCCWXH", # Long covered call writing (long stock, short call)
    "LPPBXL", "LPPBX", "LPPBXH", # Long protective put buying (long stock, long call)
    "LLCXL", "LLCX", "LLCXH", # Long leveraged call (long stock, long call)
    "LLPXL", "LLPX", "LLPXH" # Long leveraged put (Long stock, short put)
  )
#
# VaR analysis of European-style options
#
  inputStyle = 1  # 1 for European-style, 2 for American-style
  IV <- fInitialValues(BINInputData, InitialValuations) # NOTE: This function will change with option models
#
# Set up simulations
#
# Test normal distribution (Normal with mean = Mean and
#   standard deviation = StandardDeviation)
# Sample (s)
  NSim <- SimulationInputData$NumberOfSimulations
# If simulated covariance is not positive definite, then adjust  
  if(!is.positive.definite(Covariance)){ # Is Dmat not positive definite?
    pd_D_mat <- nearPD(Covariance)  # Find nearby positive definite solution
    Covariance = as.matrix(pd_D_mat$mat)
    Errorpd_D_mat <- pd_D_mat
  }
  s = mvrnorm(n = NSim, Mean, Covariance, tol = 1e-6, empirical = FALSE, EISPACK = FALSE)
  sMean = as.numeric(c(1:NumberOfRandomVariables))
  sStandardDeviation = as.numeric(c(1:NumberOfRandomVariables))
  sCorrelation = matrix(nrow = NumberOfRandomVariables, ncol = NumberOfRandomVariables)
  for(i in 1:NumberOfRandomVariables){
    sMean[i] = mean(s[,i])
    sStandardDeviation[i] = sd(s[,i])
    for(j in 1:i){
      sCorrelation[i, j] = cor(s[,i],s[,j])
      sCorrelation[j, i] = sCorrelation[i, j]
    }
  }
  TerminalValuations = data.frame(matrix(NA, nrow = NumberOfStrategies, ncol = NSim))
  colnames(TerminalValuations)  <- seq(1:NSim)
  rownames(TerminalValuations)  <- c("LS", # Long stock
    "LCXL", "LCX", "LCXH", # Long call
    "LPXL", "LPX", "LPXH", # Long put
    "LCCWXL", "LCCWX", "LCCWXH", # Long covered call writing (long stock, short call)
    "LPPBXL", "LPPBX", "LPPBXH", # Long protective put buying (long stock, long call)
    "LLCXL", "LLCX", "LLCXH", # Long leveraged call (long stock, long call)
    "LLPXL", "LLPX", "LLPXH" # Long leveraged put (Long stock, short put)
  )
  Horizon <- SimulationInputData$Horizon
  tStockPrice <- BINInputData$StockPrice
  tInterestRate <- BINInputData$InterestRate
  tVolatility <- BINInputData$Volatility
  tTimeToMaturity <- BINInputData$TimeToMaturity
  BINInputData$TimeToMaturity <- tTimeToMaturity - Horizon 
  for(i in 1:NSim){
    BINInputData$StockPrice <- tStockPrice*exp(s[i,1]*Horizon)  
    BINInputData$InterestRate <- tInterestRate*exp(s[i,2]*Horizon)
    BINInputData$Volatility <- tVolatility + s[i,3]*Horizon
    TerminalValuations[,i] <- fInitialValues(BINInputData, InitialValuations)
  }
  BINInputData$TimeToMaturity <- tTimeToMaturity
  BINInputData$StockPrice <- tStockPrice
  BINInputData$InterestRate <- tInterestRate
  BINInputData$Volatility <- tVolatility
#
# Compute VaR for each strategy
# 
  TerminalValue <- rep(-99, 19)
  Valuations <- cbind(IV, TerminalValue)
  VaR <- rep(-99, 19)
  Valuations <- cbind(Valuations, VaR)
  RVaR <- rep(-99, 19)
  Valuations <- cbind(Valuations, RVaR)
  for(i in 1:NumberOfStrategies){
    tVector <- as.matrix(TerminalValuations[i,])
    VaRTV <- as.numeric(quantile(tVector, 1-ConfidenceLevel/100))
    Valuations[i,2] <- as.numeric(VaRTV)
    Valuations[i,3] <- -(as.numeric(Valuations[i,2]) - as.numeric(Valuations[i,1])) 
    Valuations[i,4] <- (Valuations[i,3]/Valuations[i,1])*100.0
  }
  Valuations <- round(Valuations,2)  
# Record base case information  
  if(BASECASE){
    Valuations <- cbind(Strategy = rownames(Valuations), Valuations)
    rownames(Valuations) <- 1:nrow(Valuations)
    Tab <- "VaR Output"
    wb <- buildWorkbook(Valuations, sheetName = Tab, gridLines = TRUE)
# Input option data
    Tab <- "Option Parameters"
    addWorksheet(wb, sheetName = Tab, gridLines = TRUE)
    OptionInputData <- t(OptionInputData)
    OptionInputData <- cbind(Strategy = rownames(OptionInputData), OptionInputData)
    rownames(OptionInputData) <- 1:nrow(OptionInputData)
    writeData(wb, Tab, OptionInputData)
# Input simulation data
    Tab <- "Simulation Parameters"
    addWorksheet(wb, sheetName = Tab, gridLines = TRUE)
    SimulationInputData <- as.data.frame(SimulationInputData)
    SimulationInputData <- t(SimulationInputData)
    SimulationInputData <- cbind(Parameters = rownames(SimulationInputData), SimulationInputData)
    rownames(SimulationInputData) <- 1:nrow(SimulationInputData)
    writeData(wb, Tab, SimulationInputData)
#
# Build data frame to contain both input simulation parameters as well as output
#
    dfMean <- as.data.frame(Mean)
    colnames(dfMean)  <- c("PopMean")
    dfSMean <- as.data.frame(sMean)
    colnames(dfSMean)  <- c("SamMean")
    dfStandardDeviation <- as.data.frame(StandardDeviation)
    colnames(dfStandardDeviation)  <- c("PopSD")
    dfSStandardDeviation <- as.data.frame(sStandardDeviation)
    colnames(dfSStandardDeviation)  <- c("SamSD")
    dfCorrelation <- as.data.frame(Correlation)
    colnames(dfCorrelation)  <- c("PopCor1", "PopCor2", "PopCor3")
    dfSCorrelation <- as.data.frame(sCorrelation)
    colnames(dfSCorrelation)  <- c("SamCor1", "SamCor2", "SamCor3")
    dfMean <- dfMean*100
    dfSMean <- dfSMean*100
    dfStandardDeviation <- dfStandardDeviation*100
    dfSStandardDeviation <- dfSStandardDeviation*100
    SimParameters <- cbind(dfMean, dfSMean, dfStandardDeviation, dfSStandardDeviation,
      dfCorrelation, dfSCorrelation)
    SimParameters <- round(SimParameters, 2)
    rownames(SimParameters)  <- c("Stock", "Rate", "Volatililty")
    SimParameters <- cbind(Parameters = rownames(SimParameters), SimParameters)
    Tab <- "Simulation Statistics"
    addWorksheet(wb, sheetName = Tab, gridLines = TRUE)
    writeData(wb, Tab, SimParameters)
    FilePath <- "Selected Output/"
    WorkBookName <- paste0(FilePath, 'ABM BOVM VaR and Valuations ', Variable, ' ', BINInputData$Style, '.xlsx')
    saveWorkbook(wb, WorkBookName, overwrite = TRUE)
  }
  return(Valuations)
}
