# GBM OVM VCVaR Functions.R
#
source("GBM OVM With SRM Functions.R")
#
# Portfolio risk management functions
#
# Select between European-style and American-style
fGBMOptionValue = function(B){
  OV <- GBMOptionValue(B)
  return(OV)
}
# Set initial values for 19 positions
fInitialValues = function(GBMInputData, InitialValuations){
  tIV <- InitialValuations
# Long Stock
  tIV["LS", "InitialValue"] <- GBMInputData$Units * GBMInputData$StockPrice
# Long call  
  GBMInputData$Type <- 1L # 1 for call, -1 for put
  tX <- GBMInputData$StrikePrice
  tIV["LCX", "InitialValue"] <- GBMInputData$Units * fGBMOptionValue(GBMInputData)
  GBMInputData$StrikePrice <- tX*(1 - GBMInputData$StrikeIncrement/100)
  tIV["LCXL", "InitialValue"] <- GBMInputData$Units * fGBMOptionValue(GBMInputData)
  GBMInputData$StrikePrice <- tX*(1 + GBMInputData$StrikeIncrement/100)
  tIV["LCXH", "InitialValue"] <- GBMInputData$Units * fGBMOptionValue(GBMInputData)
  GBMInputData$StrikePrice <- tX
# Long put
  GBMInputData$Type <- -1L # 1 for call, -1 for put
  tX <- GBMInputData$StrikePrice
  tIV["LPX", "InitialValue"] <- GBMInputData$Units * fGBMOptionValue(GBMInputData)
  GBMInputData$StrikePrice <- tX*(1 - GBMInputData$StrikeIncrement/100)
  tIV["LPXL", "InitialValue"] <- GBMInputData$Units * fGBMOptionValue(GBMInputData)
  GBMInputData$StrikePrice <- tX*(1 + GBMInputData$StrikeIncrement/100)
  tIV["LPXH", "InitialValue"] <- GBMInputData$Units * fGBMOptionValue(GBMInputData)
  GBMInputData$StrikePrice <- tX
# Long covered call writing  
  tIV["LCCWX", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LCX", "InitialValue"]
  tIV["LCCWXL", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LCXL", "InitialValue"]
  tIV["LCCWXH", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LCXH", "InitialValue"]
# Long protective put buying
  tIV["LPPBX", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LPX", "InitialValue"]
  tIV["LPPBXL", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LPXL", "InitialValue"]
  tIV["LPPBXH", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LPXH", "InitialValue"]
# Long leveraged call  
  tIV["LLCX", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LCX", "InitialValue"]
  tIV["LLCXL", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LCXL", "InitialValue"]
  tIV["LLCXH", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LCXH", "InitialValue"]
# Long leveraged put
  tIV["LLPX", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LPX", "InitialValue"]
  tIV["LLPXL", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LPXL", "InitialValue"]
  tIV["LLPXH", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LPXH", "InitialValue"]
  tIV <- round(tIV,4)
  return(tIV)
}
