# GBM Option MCVaR Setup.R
# Stock 1
inputName <- 'ABC'
inputUnits <- 100 # Number of units (e.g., Shares)
inputStockPrice1 <- 100.0        # In dollars per share
inputStrikePrice1 <- inputStockPrice1 # In dollars per share, at-the-money assumed
inputInterestRate <- 5.0         # In percent, annualized risk-free interest rate,continuously compounded 
inputDividendYield1 <- 0.0       # In percent, continuously compounded, annualized
inputImpliedVolatility1 <- 30.0  # In percent, current implied volatility used to compute current option prices
inputTimeToMaturity1 = 1         # In years, for both puts and calls
inputType <- 1L                  # 1 for call, -1 for put
inputStrikeIncrement <- 10.0   # Percentage change to compute XL and XH 
#
# GBM list
#
GBMInputData <- list(inputName, inputUnits, inputStockPrice1, inputStrikePrice1,
  inputInterestRate, inputDividendYield1, inputImpliedVolatility1, inputTimeToMaturity1, 
  inputType, inputStrikeIncrement)
names(GBMInputData) <- c("Name", "Units", "StockPrice", "StrikePrice", "InterestRate", 
  "DividendYield", "Volatility", "TimeToMaturity", "Type", "StrikeIncrement")
OptionInputData <- as.data.frame(GBMInputData) # Used later to track inputs
# Stock 2 (eventually build out, or make assignment)

# Similuation input (only one stock for now)
inputHorizon <- 1/12 # Default (1 Week), must be less than TimeToMaturity
inputConfidenceLevel <- 95.0 # Percent
inputNumberOfSimulations <- 10000 # 10,000 ? default
inputStockMean <- 5.0     # In percent, expected percentage change in stock, continuously compounded, annualized
inputInterestRateMean <- 0.0    # In percent, expected percentage change in interest rate, cc
inputSDMean <- 0.0    # In percent, expected percentage change in SD, cc
inputStockSD <- 30.0      # In percent, continuously compounded, annualized
inputInterestRateSD <- 10.0      # In percent, continuously compounded, annualized
inputSDSD <- 40.0      # In percent, continuously compounded, annualized
inputCorRateVol <- 0.0  # Correlation between changes in rates and changes in vol
inputCorStockRate <- -0.3 # Correlation between stock returns and changes in rates
inputCorStockVol <- -0.5  # Correlation between stock returns and changes in SD
SimulationInputData <- list(inputName, inputHorizon, inputConfidenceLevel, 
  inputNumberOfSimulations, inputStockMean, inputInterestRateMean, 
  inputSDMean, inputStockSD, inputInterestRateSD,
  inputSDSD, inputCorRateVol, inputCorStockRate, inputCorStockVol)
names(SimulationInputData) <- c("Name", "Horizon", "ConfidenceLevel", 
  "NumberOfSimulations", "StockMean", "InterestRateMean", 
  "SDMean", "StockSD", "InterestRateSD", 
  "SDSD", "CorRateVol", "CorStockRate", "CorStockVol")
MCVaROutput = data.frame(matrix(NA, nrow = NumberOfStrategies, ncol = NumberOfCorrelations))
colnames(MCVaROutput)  <- seq(1:NumberOfCorrelations)
rownames(MCVaROutput)  <- c("LS", # Long stock
  "LCXL", "LCX", "LCXH", # Long call
  "LPXL", "LPX", "LPXH", # Long put
  "LCCWXL", "LCCWX", "LCCWXH", # Long covered call writing (long stock, short call)
  "LPPBXL", "LPPBX", "LPPBXH", # Long protective put buying (long stock, long call)
  "LLCXL", "LLCX", "LLCXH", # Long leveraged call (long stock, long call)
  "LLPXL", "LLPX", "LLPXH" # Long leveraged put (Long stock, short put)
)         
MCRVaROutput <- MCVaROutput

