# ABM BOVM MCVaR Functions.R
#
source("ABM OVM With SRM Functions.R")
#
# Portfolio risk management functions
#
# Select between European-style and American-style
fABMOptionValue = function(B){
  OV <- ABMOptionValue(B)
  return(OV)
}
# Set initial values for 19 positions
fInitialValues = function(ABMInputData, InitialValuations){
  tIV <- InitialValuations
# Long Stock
  tIV["LS", "InitialValue"] <- ABMInputData$Units * ABMInputData$StockPrice
# Long call  
  ABMInputData$Type <- 1L # 1 for call, -1 for put
  tX <- ABMInputData$StrikePrice
  tIV["LCX", "InitialValue"] <- ABMInputData$Units * fABMOptionValue(ABMInputData)
  ABMInputData$StrikePrice <- tX*(1 - ABMInputData$StrikeIncrement/100)
  tIV["LCXL", "InitialValue"] <- ABMInputData$Units * fABMOptionValue(ABMInputData)
  ABMInputData$StrikePrice <- tX*(1 + ABMInputData$StrikeIncrement/100)
  tIV["LCXH", "InitialValue"] <- ABMInputData$Units * fABMOptionValue(ABMInputData)
  ABMInputData$StrikePrice <- tX
  # Long put
  ABMInputData$Type <- -1L # 1 for call, -1 for put
  tX <- ABMInputData$StrikePrice
  tIV["LPX", "InitialValue"] <- ABMInputData$Units * fABMOptionValue(ABMInputData)
  ABMInputData$StrikePrice <- tX*(1 - ABMInputData$StrikeIncrement/100)
  tIV["LPXL", "InitialValue"] <- ABMInputData$Units * fABMOptionValue(ABMInputData)
  ABMInputData$StrikePrice <- tX*(1 + ABMInputData$StrikeIncrement/100)
  tIV["LPXH", "InitialValue"] <- ABMInputData$Units * fABMOptionValue(ABMInputData)
  ABMInputData$StrikePrice <- tX
  # Long covered call writing  
  tIV["LCCWX", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LCX", "InitialValue"]
  tIV["LCCWXL", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LCXL", "InitialValue"]
  tIV["LCCWXH", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LCXH", "InitialValue"]
  # Long protective put buying
  tIV["LPPBX", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LPX", "InitialValue"]
  tIV["LPPBXL", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LPXL", "InitialValue"]
  tIV["LPPBXH", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LPXH", "InitialValue"]
  # Long leveraged call  
  tIV["LLCX", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LCX", "InitialValue"]
  tIV["LLCXL", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LCXL", "InitialValue"]
  tIV["LLCXH", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LCXH", "InitialValue"]
  # Long leveraged put
  tIV["LLPX", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LPX", "InitialValue"]
  tIV["LLPXL", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LPXL", "InitialValue"]
  tIV["LLPXH", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LPXH", "InitialValue"]
  tIV <- round(tIV,4)
  return(tIV)
}  
  
  
  
  
# # Long call  
#   ABMInputData$Type <- 1L # 1 for call, -1 for put
#   tX <- ABMInputData$StrikePrice
#   Value <-fABMOptionValue(ABMInputData)
#   tIV["LCX", "InitialValue"] <- ABMInputData$Units * Value
#   ABMInputData$StrikePrice <- tX*(1 - ABMInputData$StrikeIncrement/100)
#   Value <-fABMOptionValue(ABMInputData)
#   tIV["LCXL", "InitialValue"] <- ABMInputData$Units * Value
#   ABMInputData$StrikePrice <- tX*(1 + ABMInputData$StrikeIncrement/100)
#   Value <-fABMOptionValue(ABMInputData)
#   tIV["LCXH", "InitialValue"] <- ABMInputData$Units * Value
#   ABMInputData$StrikePrice <- tX
# # Long put
#   ABMInputData$Type <- -1L # 1 for call, -1 for put
#   tX <- ABMInputData$StrikePrice
#   Value <-fABMOptionValue(ABMInputData)
#   tIV["LPX", "InitialValue"] <- ABMInputData$Units * Value
#   ABMInputData$StrikePrice <- tX*(1 - ABMInputData$StrikeIncrement/100)
#   Value <-fABMOptionValue(ABMInputData)
#   tIV["LPXL", "InitialValue"] <- ABMInputData$Units * Value
#   ABMInputData$StrikePrice <- tX*(1 + ABMInputData$StrikeIncrement/100)
#   Value <-fABMOptionValue(ABMInputData)
#   tIV["LPXH", "InitialValue"] <- ABMInputData$Units * Value
#   ABMInputData$StrikePrice <- tX
# # Long covered call writing  
#   tIV["LCCWX", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LCX", "InitialValue"]
#   tIV["LCCWXL", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LCXL", "InitialValue"]
#   tIV["LCCWXH", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LCXH", "InitialValue"]
# # Long protective put buying
#   tIV["LPPBX", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LPX", "InitialValue"]
#   tIV["LPPBXL", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LPXL", "InitialValue"]
#   tIV["LPPBXH", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LPXH", "InitialValue"]
# # Long leveraged call  
#   tIV["LLCX", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LCX", "InitialValue"]
#   tIV["LLCXL", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LCXL", "InitialValue"]
#   tIV["LLCXH", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LCXH", "InitialValue"]
# # Long leveraged put
#   tIV["LLPX", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LPX", "InitialValue"]
#   tIV["LLPXL", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LPXL", "InitialValue"]
#   tIV["LLPXH", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LPXH", "InitialValue"]
#   tIV <- round(tIV,4)
#   return(tIV)
# }
