# GBM OVM VCVaR Functions.R
#
source("GBM COVM w Greeks Functions.R")
#
# Portfolio risk management functions
#
# Select between European-style and American-style
fCOValue = function(COInputData){
  LowerBound <- 0
  UpperBound <- 5.0*COInputData$S #Critical stock price for put has 2 solutions
  OV <- COValue(COInputData, LowerBound, UpperBound)
  return(OV)
}
# Set initial values for 19 positions
fInitialValues = function(COInputData, InitialValuations){
  tIV <- InitialValuations
# Long Stock
# Derive BSMOVM list
  inputUnderlying <- COInputData$S
  inputUnderlyingStrikePrice <- COInputData$XU
  inputInterestRate <- COInputData$r
  inputUnderlyingYield <- COInputData$d
  inputOptionYield <- COInputData$q
  inputVolatility <- COInputData$v
  inputTimeToMaturity <- COInputData$TU
  inputType <- COInputData$iU
  BSMInputData <- list(inputUnderlying, inputUnderlyingStrikePrice, 
    inputInterestRate, inputUnderlyingYield,  inputOptionYield, 
    inputVolatility, inputTimeToMaturity, inputType)
  names(BSMInputData) <- c("StockPrice", "StrikePrice", "InterestRate",
    "DividendYield", "OptionYield", "Volatility", "TimeToMaturity", "Type")  
  tIV["LS", "InitialValue"] <- COInputData$Units * BSMOptionValue(BSMInputData)
# Long call  
  COInputData$iC <- 1L # 1 for call, -1 for put
  COInputData$iU <- 1L # 1 on call, -1 on put
  tX <- COInputData$XU
  tIV["LCX", "InitialValue"] <- COInputData$Units * fCOValue(COInputData)
  COInputData$XU <- tX*(1 - COInputData$StrikeIncrement/100)
  tIV["LCXL", "InitialValue"] <- COInputData$Units * fCOValue(COInputData)
  COInputData$XU <- tX*(1 + COInputData$StrikeIncrement/100)
  tIV["LCXH", "InitialValue"] <- COInputData$Units * fCOValue(COInputData)
  COInputData$XU <- tX
# Long put
  COInputData$iU <- -1L # 1 on call, -1 on put
  tX <- COInputData$XU
  tIV["LPX", "InitialValue"] <- COInputData$Units * fCOValue(COInputData)
  COInputData$XU <- tX*(1 - COInputData$StrikeIncrement/100)
  tIV["LPXL", "InitialValue"] <- COInputData$Units * fCOValue(COInputData)
  COInputData$XU <- tX*(1 + COInputData$StrikeIncrement/100)
  tIV["LPXH", "InitialValue"] <- COInputData$Units * fCOValue(COInputData)
  COInputData$XU <- tX
# Long covered call writing  
  tIV["LCCWX", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LCX", "InitialValue"]
  tIV["LCCWXL", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LCXL", "InitialValue"]
  tIV["LCCWXH", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LCXH", "InitialValue"]
# Long protective put buying
  tIV["LPPBX", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LPX", "InitialValue"]
  tIV["LPPBXL", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LPXL", "InitialValue"]
  tIV["LPPBXH", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LPXH", "InitialValue"]
# Long leveraged call  
  tIV["LLCX", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LCX", "InitialValue"]
  tIV["LLCXL", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LCXL", "InitialValue"]
  tIV["LLCXH", "InitialValue"] <- tIV["LS", "InitialValue"] +  tIV["LCXH", "InitialValue"]
# Long leveraged put
  tIV["LLPX", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LPX", "InitialValue"]
  tIV["LLPXL", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LPXL", "InitialValue"]
  tIV["LLPXH", "InitialValue"] <- tIV["LS", "InitialValue"] -  tIV["LPXH", "InitialValue"]
  tIV <- round(tIV,4)
  return(tIV)
}
