# GBM COVM MCVaR Test.R
rm(list = ls())  # Take out the Environment "trash"
cat("\014") # Clear Console, making error checking easier.
while (!is.null(dev.list()))  dev.off() # Clear old plots
par(family = 'Times New Roman') # Globally set fonts for graphs
# Libraries
#  MASS - Multivariate normal random number generator
Packages <- c("MASS","openxlsx", "Matrix", "matrixcalc", "beepr", "mvtnorm")
if(length(setdiff(Packages, rownames(installed.packages()))) > 0) {
  install.packages(setdiff(Packages, rownames(installed.packages())))
} # Make sure libraries are installed on this computer
lapply(Packages, library, character.only=TRUE) # Load and attach libraries
rm(Packages)
# Track runtime
op <- options(digits.secs = 6)
StartTime <- Sys.time()
LagtEndRunTime <- StartTime
# Build data frames of inputs
# Note:
#      w = weight of strike price
#      X = (1 - w)S, S, (1 + w)S of par (evaluate only three)
# 19 Strategies compared:
#   1) Long N stock
#   2) Long N calls (XL, X, XH)
#   3) Long N puts (XL, X, XH)
#   4) Covered call writing (long N stock, short N calls) (XL, X, XH)
#   5) Protective put buying (long N stock, long N puts) (XL, X, XH)
#   6) Leveraged call buying (long N stock, long N calls) (XL, X, XH)
#   7) Leveraged put selling (long N stock, short N puts) (XL, X, XH)
#   
#
# Study #1: Role of correlation between Stock and Volatility
#
LowerBound <- -0.75
UpperBound <- 0.75
NumberOfStrategies <- 19 # Long only for now
NumberOfCorrelations <- 7
source('GBM COVM MCVaR Core.R')
#
# Source all required functions
#
source("GBM COVM MCVaR Functions.R")
# Loop over all cases
#   Variable <- "CorStockVol"
#   Variable <- "CorStockRate"
#   Variable <- "CorRateVol"
#.  With Style = 1 ES or 2 AS
for(kk in 1:3){
  if(kk == 1){
    source('GBM COption MCVaR Setup.R')
    Variable <- "CorStockVol"
  } else if(kk == 2){
    source('GBM COption MCVaR Setup.R')
    Variable <- "CorStockRate"
  } else {
    source('GBM COption MCVaR Setup.R')
    Variable <- "CorRateVol"
  }
  # Sensitivity Variable  
  Increment <- (UpperBound - LowerBound)/(NumberOfCorrelations - 1)
  SensitivityVariable = data.frame(matrix(NA, nrow = NumberOfCorrelations, ncol = 1))
  colnames(SensitivityVariable)  <- 'Parameter'
  rownames(SensitivityVariable)  <- seq(1:NumberOfCorrelations)
  for(i in 1:NumberOfCorrelations){
    SensitivityVariable$Parameter[i] <- LowerBound + (i - 1)*Increment
  }
  Tab <- "Sensitivity Variable"
  SensitivityVariable <- cbind(Parameters = rownames(SensitivityVariable), SensitivityVariable)
  rownames(SensitivityVariable) <- 1:nrow(SensitivityVariable)
  wb <- buildWorkbook(SensitivityVariable, sheetName = Tab, gridLines = TRUE)
  BASECASE <- FALSE
  if(Variable == "CorStockVol"){
    tCorStockVol <- SimulationInputData$CorStockVol
  }
  if(Variable == "CorStockRate"){
    tCorStockRate <- SimulationInputData$CorStockRate
  }
  if(Variable == "CorRateVol"){
    tCorRateVol <- SimulationInputData$CorRateVol
  }
  for(i in 1:NumberOfCorrelations){
    if(Variable == "CorStockVol")SimulationInputData$CorStockVol <- LowerBound + (i - 1)*Increment
    if(Variable == "CorStockRate")SimulationInputData$CorStockRate <- LowerBound + (i - 1)*Increment
    if(Variable == "CorRateVol")SimulationInputData$CorRateVol <- LowerBound + (i - 1)*Increment
    tMCOutput <- fGBMCOVMMCVaRCore(COInputData, InitialValuations, Variable)
    MCVaROutput[,i] <- tMCOutput$VaR
    MCRVaROutput[,i] <- tMCOutput$RVaR
  }
  if(Variable == "CorStockVol"){
    SimulationInputData$CorStockVol <- tCorStockVol
  }
  if(Variable == "CorStockRate"){
    SimulationInputData$CorStockRate <- tCorStockRate
  }
  if(Variable == "CorRateVol"){
    SimulationInputData$CorRateVol <- tCorRateVol
  }
  # VaR output based on sensitivities
  MCVaROutput <- cbind(Strategy = rownames(MCVaROutput), MCVaROutput)
  rownames(MCVaROutput) <- 1:nrow(MCVaROutput)
  Tab <- "MCVaR Output"
  addWorksheet(wb, sheetName = Tab, gridLines = TRUE)
  writeData(wb, Tab, MCVaROutput)
  # Return VaR output based on sensitivities
  MCRVaROutput <- cbind(Strategy = rownames(MCRVaROutput), MCRVaROutput)
  rownames(MCRVaROutput) <- 1:nrow(MCRVaROutput)
  Tab <- "MCRVaR Output"
  addWorksheet(wb, sheetName = Tab, gridLines = TRUE)
  writeData(wb, Tab, MCRVaROutput)
  FilePath <- "Selected Output/"
  WorkBookName <- paste0(FilePath, 'GBM BOVM MC VaR and RVaR ', Variable, ' 1', '.xlsx')
  saveWorkbook(wb, WorkBookName, overwrite = TRUE)
  BASECASE <- TRUE
  Valuations <- fGBMCOVMMCVaRCore(COInputData, InitialValuations, Variable)
  tEndRunTime <- Sys.time()
  inputNumberOfSimulations
  tRunTime <- tEndRunTime - LagtEndRunTime
  LagtEndRunTime <- tEndRunTime
  print(tRunTime)
}
EndTime <- Sys.time()
inputNumberOfSimulations
RunTime <- EndTime - StartTime
RunTime
beep(sound = 2, print('Finished'))


# rm(list = ls())  # Take out the Environment "trash"
# cat("\014") # Clear Console, making error checking easier.
# while (!is.null(dev.list()))  dev.off() # Clear old plots
# par(family = 'Times New Roman') # Globally set fonts for graphs
# # Libraries
# #  MASS - Multivariate normal random number generator
# Packages <- c("MASS","openxlsx", "pracma", "mvtnorm", "pbivnorm")
# if(length(setdiff(Packages, rownames(installed.packages()))) > 0) {
#   install.packages(setdiff(Packages, rownames(installed.packages())))
# } # Make sure libraries are installed on this computer
# lapply(Packages, library, character.only=TRUE) # Load and attach libraries
# rm(Packages)
# # Build data frames of inputs
# # Note:
# #      w = weight of strike price
# #      X = (1 - w)S, S, (1 + w)S of par (evaluate only three)
# # 19 Strategies compared:
# #   1) Long N stock
# #   2) Long N calls (XL, X, XH)
# #   3) Long N puts (XL, X, XH)
# #   4) Covered call writing (long N stock, short N calls) (XL, X, XH)
# #   5) Protective put buying (long N stock, long N puts) (XL, X, XH)
# #   6) Leveraged call buying (long N stock, long N calls) (XL, X, XH)
# #   7) Leveraged put selling (long N stock, short N puts) (XL, X, XH)
# #   
# #
# # Study #1: Role of correlation between Stock and Volatility
# #
# # Variable <- "CorStockVol"
# Variable <- "CorRateVol"
# LowerBound <- -0.9
# UpperBound <- 0.9
# NumberOfStrategies <- 19 # Long only for now
# source('GBM COption MCVaR Setup.R')
# source('GBM COVM MCVaR Core.R')
# #
# # Sensitivity Variable    
# #  Assume sensitivity loop is same count as number of strategies
# #
# SensitivityVariable = data.frame(matrix(NA, nrow = NumberOfStrategies, ncol = 1))
# colnames(SensitivityVariable)  <- 'Parameter'
# rownames(SensitivityVariable)  <- seq(1:NumberOfStrategies)
# for(i in 1:NumberOfStrategies){
#   SensitivityVariable$Parameter[i] <- LowerBound + (i - 1)*Increment
# }
# Tab <- "Sensitivity Variable"
# SensitivityVariable <- cbind(Parameters = rownames(SensitivityVariable), SensitivityVariable)
# rownames(SensitivityVariable) <- 1:nrow(SensitivityVariable)
# wb <- buildWorkbook(SensitivityVariable, sheetName = Tab, gridLines = TRUE)
# BASECASE <- FALSE
# for(i in 1:NumberOfStrategies){
#   SimulationInputData$CorStockVol <- LowerBound + (i - 1)*Increment
#   tMCOutput <- fGBMCOVMMCVaRCore(COInputData, InitialValuations, Variable)
#   MCVaROutput[,i] <- tMCOutput$VaR
#   MCRVaROutput[,i] <- tMCOutput$RVaR
# }
# # VaR output based on sensitivities
# MCVaROutput <- cbind(Strategy = rownames(MCVaROutput), MCVaROutput)
# rownames(MCVaROutput) <- 1:nrow(MCVaROutput)
# Tab <- "MCVaR Output"
# addWorksheet(wb, sheetName = Tab, gridLines = TRUE)
# writeData(wb, Tab, MCVaROutput)
# # Return VaR output based on sensitivities
# MCRVaROutput <- cbind(Strategy = rownames(MCRVaROutput), MCRVaROutput)
# rownames(MCRVaROutput) <- 1:nrow(MCRVaROutput)
# Tab <- "MCRVaR Output"
# addWorksheet(wb, sheetName = Tab, gridLines = TRUE)
# writeData(wb, Tab, MCRVaROutput)
# WorkBookName <- paste0('MC VaR and RVaR ', Variable, '.xlsx')
# saveWorkbook(wb, WorkBookName, overwrite = TRUE)
# BASECASE <- TRUE
# Valuations <- fGBMCOVMMCVaRCore(COInputData, InitialValuations, Variable)

