# GBM COption MCVaR Setup.R
# Stock 1
inputName <- 'ABC'
inputUnits <- 100 # Number of units (e.g., Shares)
inputiC <- 1                              # 1-CO call, -1-CO put
inputiU <- 1                              # 1-UO call, -1-UO put
inputUnderlying <- 100.0                  # Must be positive
inputUnderlyingStrikePrice <- 100.0      # Must be positive
inputCompoundStrikePrice <- 20.46  # Must be positive
inputInterestRate <- 5.0         # In percent, annualized risk-free interest rate,continuously compounded 
inputUnderlyingYield <- 5.0              # Payout of underlying instrument
inputOptionYield <- 0.0 #log(1.05)*100                    # Payout of option
inputImpliedVolatility <- 30.0  # In percent, current implied volatility used to compute current option prices
inputUnderlyingTimeToMaturity <- 5.0     # Must be positive
inputCompoundTimeToMaturity <- 1.0        # Must be positive
inputStrikeIncrement <- 10.0   # Percentage change to compute XL and XH 
inputGreekIncrement <- 1.0   # Greek increment
#
# GBM Compound Option List
#
COInputData <- list(inputName, inputUnits, inputiC, inputiU, inputUnderlying, 
  inputUnderlyingStrikePrice, inputCompoundStrikePrice, inputInterestRate, 
  inputUnderlyingYield, inputOptionYield, inputImpliedVolatility, inputUnderlyingTimeToMaturity, 
  inputCompoundTimeToMaturity, inputGreekIncrement, inputStrikeIncrement)
names(COInputData) <- c("Name", "Units", "iC", "iU", "S", "XU", "XC", "r", "d", 
  "q", "v", "TU", "TC", "Increment", "StrikeIncrement")
# Derive BSMOVM list
inputUnderlying <- COInputData$S
inputUnderlyingStrikePrice <- COInputData$XU
inputInterestRate <- COInputData$r
inputUnderlyingYield <- COInputData$d
inputOptionYield <- COInputData$q
inputVolatility <- COInputData$v
inputTimeToMaturity <- COInputData$TU
inputType <- COInputData$iU
BSMInputData <- list(inputUnderlying, inputUnderlyingStrikePrice, 
  inputInterestRate, inputUnderlyingYield,  inputOptionYield, 
  inputVolatility, inputTimeToMaturity, inputType)
names(BSMInputData) <- c("StockPrice", "StrikePrice", "InterestRate",
  "DividendYield", "OptionYield", "Volatility", "TimeToMaturity", "Type")
OptionInputData <- as.data.frame(COInputData) # Used later to track inputs
# Similuation input (only one stock for now)
inputHorizon <- 1/12 # Default (1 Week), must be less than TimeToMaturity
inputConfidenceLevel <- 95.0 # Percent
inputNumberOfSimulations <- 10000
inputStockMean <- 5.0     # In percent, expected percentage change in stock, continuously compounded, annualized
inputInterestRateMean <- 0.0    # In percent, expected percentage change in interest rate, cc
inputSDMean <- 0.0    # In percent, expected percentage change in SD, cc
inputStockSD <- 30.0      # In percent, continuously compounded, annualized
inputInterestRateSD <- 10.0      # In percent, continuously compounded, annualized
inputSDSD <- 40.0      # In percent, continuously compounded, annualized
inputCorRateVol <- 0.0  # Correlation between changes in rates and changes in vol
inputCorStockRate <- -0.3 # Correlation between stock returns and changes in rates
inputCorStockVol <- -0.5  # Correlation between stock returns and changes in SD
SimulationInputData <- list(inputName, inputHorizon, inputConfidenceLevel, 
  inputNumberOfSimulations, inputStockMean, inputInterestRateMean, 
  inputSDMean, inputStockSD, inputInterestRateSD,
  inputSDSD, inputCorRateVol, inputCorStockRate, inputCorStockVol)
names(SimulationInputData) <- c("Name", "Horizon", "ConfidenceLevel", 
  "NumberOfSimulations", "StockMean", "InterestRateMean", 
  "SDMean", "StockSD", "InterestRateSD", 
  "SDSD", "CorRateVol", "CorStockRate", "CorStockVol")
MCVaROutput = data.frame(matrix(NA, nrow = NumberOfStrategies, ncol = NumberOfCorrelations))
colnames(MCVaROutput)  <- seq(1:NumberOfCorrelations)
rownames(MCVaROutput)  <- c("LS", # Long stock
  "LCXL", "LCX", "LCXH", # Long call
  "LPXL", "LPX", "LPXH", # Long put
  "LCCWXL", "LCCWX", "LCCWXH", # Long covered call writing (long stock, short call)
  "LPPBXL", "LPPBX", "LPPBXH", # Long protective put buying (long stock, long call)
  "LLCXL", "LLCX", "LLCXH", # Long leveraged call (long stock, long call)
  "LLPXL", "LLPX", "LLPXH" # Long leveraged put (Long stock, short put)
)         
MCRVaROutput <- MCVaROutput

